import React, { FC, useState } from 'react';
import { Transition } from 'react-transition-group';
import { CardSelection } from '../models/PlayableCard';
import { Button } from './Button';
import { Board } from './card/Board';
import { BottomBar, BottomBarCenter, BottomBarContainer, BottomBarLeft, BottomBarRight } from './containers/BottomBar';
import { PlayContainer, PlayContainerBot, PlayContainerTop } from './containers/PlayContainer';
import { Progress, TopBar } from './containers/TopBar';

export type MultistageCardSelectProp = {
    cards: CardSelection;
    onValidate?: (ids: number[]) => void;
    onChange?: (ids: number[]) => void;
};

const MultistageCardSelect: FC<MultistageCardSelectProp> = ({ cards, onValidate, onChange }: MultistageCardSelectProp) => {
    const duration = 100;

    const defaultStyle: React.CSSProperties = {
        transition: `transform ${duration}ms ease-in-out`,
        transform: 'translateX(0)'
    };

    const transitionStyles: { [index: string]: React.CSSProperties } = {
        entering: { transform: 'translateX(100vw)', transition: `transform 0ms ease-in-out` },
        entered: { transform: 'translateX(0)' },
        exiting: { transform: 'translateX(0)' },
        exited: { transform: 'translateX(-100vw)' }
    };

    const backTransitionStyles: { [index: string]: React.CSSProperties } = {
        entering: { transform: 'translateX(-100vw)', transition: `transform 0ms ease-in-out` },
        entered: { transform: 'translateX(0)' },
        exiting: { transform: 'translateX(0)' },
        exited: { transform: 'translateX(100vw)' }
    };

    const [currentPage, setCurrentPage] = useState<number>(0);
    const [selection, setSelection] = useState<{ [index: number]: number[] }>({});
    const [show, setShow] = useState(true);
    const [isBack, setBack] = useState(false);

    const selectionChanged = (ids: number[]) => {
        const sel = selection;
        sel[cards[currentPage].id] = ids;
        setSelection(sel);
        if (onChange !== undefined) {
            onChange(Object.values(sel).flat());
        }
    };

    const nextPage = () => {
        setBack(false);
        setShow(false);
        setTimeout(() => {
            if (currentPage < cards.length - 1) {
                setCurrentPage(currentPage + 1);
                setShow(true);
            } else if (currentPage === cards.length - 1 && onValidate !== undefined) {
                onValidate(Object.values(selection).flat());
            }
        }, duration * 2);
    };

    const previousPage = () => {
        setBack(true);
        setShow(false);
        setTimeout(() => {
            if (currentPage > 0) {
                setCurrentPage(currentPage - 1);
            }
            setShow(true);
        }, duration * 2);
    };

    return (
        <PlayContainer>
            <PlayContainerTop>
                <TopBar>
                    <Progress percent={(currentPage / (cards.length - 1)) * 100} />
                </TopBar>
            </PlayContainerTop>
            <Transition timeout={duration} in={show}>
                {(state) => (
                    <div
                        className="component__playcontainer__mid"
                        style={{
                            ...defaultStyle,
                            ...(isBack ? backTransitionStyles : transitionStyles)[state]
                        }}
                    >
                        <Board
                            cardsCount={cards[currentPage].count}
                            cards={cards[currentPage].cards}
                            selected={selection[cards[currentPage].id] ?? []}
                            onSelectionChange={selectionChanged}
                        />
                    </div>
                )}
            </Transition>
            <PlayContainerBot>
                <BottomBar>
                    <BottomBarContainer>
                        <BottomBarLeft>
                            {
                                <Button onClick={previousPage} bordered={true} variation="normal" disabled={currentPage === 0}>
                                    Retour
                                </Button>
                            }
                        </BottomBarLeft>
                        <BottomBarCenter>
                            <span style={{ fontWeight: 'bold', fontSize: '18px' }}>{cards[currentPage].name}</span>
                        </BottomBarCenter>
                        <BottomBarRight>
                            {
                                <Button onClick={nextPage} bordered={false} variation="success">
                                    {currentPage === cards.length - 1 ? 'Valider' : 'Suivant'}
                                </Button>
                            }
                        </BottomBarRight>
                    </BottomBarContainer>
                </BottomBar>
            </PlayContainerBot>
        </PlayContainer>
    );
};

export default MultistageCardSelect;
