import React, { FC } from 'react';
import { hslToRgb } from '../models/colors';
import { Run } from '../store/reducer';

/**
 * Convertie un score vers une couleur
 * @param score Score
 * @returns Couleur, de rouge à vers
 */
export const scoreToColor: (score: number) => string = (score: number) => {
    if (score < -1) return 'rgb(255,0,0)';
    if (score > 1) return 'rgb(0,255,0)';

    const val = (score + 1) / 2;
    const [r, g, b] = hslToRgb((val / 360) * 120, 1, 0.8);
    return `rgb(${Math.round(r)},${Math.round(g)},${Math.round(b)})`;
};

/**
 * Propriétés de l'affichage de progression
 */
export interface ProgressionProps {
    runs: Run[];
}

/**
 * Affichage de la progression
 * @param props Propriétés
 * @returns Contenue (jsx)
 */
export const Progression: FC<ProgressionProps> = ({ runs }) => {
    const width: number = 23.375459 + (runs.length - 1) * 23.63772;
    const runCommonStyle: React.SVGProps<SVGRectElement> = {
        width: 15.468329,
        height: 15.468329,
        transform: 'rotate(45)',
        fillOpacity: 1,
        stroke: '#cfcfcf',
        strokeWidth: 1.5,
        strokeLinecap: 'round',
        strokeLinejoin: 'round',
        strokeMiterlimit: 4,
        strokeDasharray: 'none',
        strokeOpacity: 1
    };

    const runFutureCommonStyle: React.SVGProps<SVGRectElement> = {
        fillOpacity: 0,
        strokeDasharray: '1.5, 4.5'
    };

    const pathCommonStyle: React.SVGProps<SVGPathElement> = {
        fill: 'none',
        fillRule: 'evenodd',
        stroke: '#cfcfcf',
        strokeWidth: 1.5,
        strokeLinecap: 'round',
        strokeLinejoin: 'miter',
        strokeMiterlimit: 4,
        strokeDasharray: 'none',
        strokeOpacity: 1
    };

    const runPathCommonStyle: React.SVGProps<SVGPathElement> = {
        strokeDasharray: '1.5, 4.5',
        strokeDashoffset: 4.2
    };

    return (
        <svg xmlns="http://www.w3.org/2000/svg" style={{ height: '10vh' }} viewBox={`0 0 ${width} 47.542412`}>
            <g transform="translate(-8.763942,-9.243386)">
                {runs.map((run, index) => {
                    return (
                        <rect
                            key={index}
                            {...runCommonStyle}
                            {...(!run.done ? runFutureCommonStyle : {})}
                            fill={scoreToColor(run.score ?? 0)}
                            x={21.902107 + Math.floor((index + 1) / 2) * 33.42878}
                            y={-7.0209241 - Math.floor(index / 2) * 33.4288289}
                        />
                    );
                })}
                {runs.map((run, index) => {
                    if (index === 0) {
                        return <React.Fragment key={index} />;
                    }

                    if (index % 2 === 1) {
                        return (
                            <path
                                key={index}
                                {...pathCommonStyle}
                                {...(!run.done ? runPathCommonStyle : {})}
                                d={`M ${25.920552 + Math.floor((index - 1) / 2) * 47.27547},26.929226 ${
                                    38.620508 + Math.floor((index - 1) / 2) * 47.27547
                                },39.629183`}
                            />
                        );
                    } else {
                        return (
                            <path
                                key={index}
                                {...pathCommonStyle}
                                {...(!run.done ? runPathCommonStyle : {})}
                                d={`M ${49.558269 + Math.floor((index - 1) / 2) * 47.27547},39.629183 ${
                                    62.258259 + Math.floor((index - 1) / 2) * 47.27547
                                },26.929226`}
                            />
                        );
                    }
                })}
            </g>
        </svg>
    );
};
