import React, { FC, useEffect, useState } from 'react';
import { PlayableCard } from '../../models/PlayableCard';
import { Card, CardDescription, CardIcon, CardTitle } from './Card';
import { Deck } from './Deck';
import { Playground } from './Playground';

/**
 * Propriétés pour la Board
 */
export interface BoardProp {
    /**
     * Nombre de cartes
     */
    cardsCount: number;

    /**
     * Cartes jouables
     */
    cards: PlayableCard[];

    /**
     * Event handler appelé quand la sélection change
     */
    onSelectionChange?: (ids: number[]) => void;

    /**
     * Cartes déjà sélectionnés
     */
    selected: number[];
}

/**
 * Propriétés par défaut pour la Board
 */
const defaultProps: BoardProp = {
    cardsCount: 3,
    cards: [],
    selected: []
};

/**
 * Espace de jeu, permettant de placer des cartes
 * @param props Propriétés du composanbt
 * @returns Contenue du composant (jsx)
 */
export const Board: FC<BoardProp> = ({ cardsCount, cards, selected: preSel, onSelectionChange }: BoardProp) => {
    const [selected, setSelected] = useState<number[]>([]);

    useEffect(() => {
        setSelected(preSel);
    }, [cardsCount, cards, preSel]);

    const deckCards: JSX.Element[] = [];
    const selectedCards: JSX.Element[] = [];

    const cardClick = (id: number) => {
        if (selected.length < cardsCount) {
            setSelected([...selected, id]);
            if (onSelectionChange !== undefined) onSelectionChange([...selected, id]);
        }
    };

    const selectedCradClick = (id: number) => {
        const selectedCopy = [...selected];
        const index = selectedCopy.indexOf(id);
        if (index > -1) {
            selectedCopy.splice(index, 1);
            setSelected(selectedCopy);
        }
        if (onSelectionChange !== undefined) onSelectionChange(selectedCopy);
    };

    for (const card of cards) {
        if (selected.includes(card.id)) continue;
        deckCards.push(
            <Card
                key={card.id}
                onClick={() => {
                    cardClick(card.id);
                }}
            >
                <CardTitle>{card.name}</CardTitle>
                {card.image === undefined ? '' : <CardIcon src={card.image} alt={card.name} />}
                <CardDescription>{card.description}</CardDescription>
            </Card>
        );
    }

    for (const id of selected) {
        const card = cards.find((obj) => obj.id === id);
        if (card !== undefined) {
            selectedCards.push(
                <Card
                    key={card.id}
                    onClick={() => {
                        selectedCradClick(card.id);
                    }}
                >
                    <CardTitle>{card.name}</CardTitle>
                    {card.image === undefined ? '' : <CardIcon src={card.image} alt={card.name} />}
                    <CardDescription>{card.description}</CardDescription>
                </Card>
            );
        }
    }

    return (
        <>
            <Playground cards_count={cardsCount}>{selectedCards}</Playground>
            <Deck>{deckCards}</Deck>
        </>
    );
};

Board.defaultProps = defaultProps;
