import React, { FC, useEffect, useState } from 'react';
import { useDispatch, useSelector } from 'react-redux';
import { useNavigate } from 'react-router-dom';
import { Transition } from 'react-transition-group';
import MultistageCardSelect from '../components/MultistageCardSelect';
import { StatsDisplay } from '../components/StatsDisplay';
import { calcEffects } from '../models/calc';
import { Cards } from '../models/cards';
import { filterCards } from '../models/filter';
import { EndEffect } from '../models/PlayableCard';
import { calcScore } from '../models/score';
import { AppState } from '../store/reducer';

/**
 * Page principale de sélection des cartes
 * @returns Contenue (jsx)
 */
export const Game: FC = () => {
    const duration = 500;

    const defaultStyle: React.CSSProperties = {
        transition: `opacity ${duration}ms ease-in-out`,
        opacity: 0
    };

    const transitionStyles: { [index: string]: React.CSSProperties } = {
        entering: { opacity: 1 },
        entered: { opacity: 1 },
        exiting: { opacity: 0 },
        exited: { opacity: 0 }
    };

    const rightDefaultStyle: React.CSSProperties = {
        transition: `right ${duration}ms ease-in-out`,
        right: '-25vw'
    };

    const rightTransitionStyles: { [index: string]: React.CSSProperties } = {
        entering: { right: '-25vw' },
        entered: { right: 0 },
        exiting: { right: 0 },
        exited: { right: '-25vw' }
    };

    const stats = useSelector((state: AppState) => state.stats);
    const [shownEffects, setShownEffects] = useState<EndEffect[]>(calcEffects([], Cards));
    const [score, setScore] = useState<number>(calcScore(calcEffects([], Cards), stats.reputation));
    const dispatch = useDispatch();
    const navigate = useNavigate();
    const [show, setShow] = useState<boolean>(false);
    const [showRight, setShowRight] = useState<boolean>(false);

    const onValidate = (val: number[]) => {
        dispatch({
            type: 'game/end',
            effects: calcEffects(val, Cards)
        });
        setShowRight(false);
    };

    const onChange = (val: number[]) => {
        const effects = calcEffects(val, Cards);
        setShownEffects(effects);
        setScore(calcScore(effects, stats.reputation));
    };

    useEffect(() => {
        setTimeout(() => setShow(true), 0);
    }, []);

    return (
        <Transition in={show} timeout={duration} onEntered={() => setShowRight(true)} onExited={() => navigate('/end')}>
            {(backState) => (
                <div
                    style={{
                        ...defaultStyle,
                        ...transitionStyles[backState]
                    }}
                >
                    <MultistageCardSelect cards={filterCards(Cards, stats)} onValidate={onValidate} onChange={onChange} />
                    <Transition in={showRight} timeout={duration} onExited={() => setShow(false)}>
                        {(state) => (
                            <div
                                className="stats__right"
                                style={{
                                    ...rightDefaultStyle,
                                    ...rightTransitionStyles[state]
                                }}
                            >
                                <StatsDisplay effects={shownEffects} score={score} />
                            </div>
                        )}
                    </Transition>
                </div>
            )}
        </Transition>
    );
};
